precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;

uniform float scaleFactor;
uniform float textureSize;
uniform float textureWidth;
uniform float textureHeight;

uniform sampler2D sTexture2;
uniform float texture2Size;
uniform float texture2Width;
uniform float texture2Height;

uniform float fade;
uniform float right;
uniform float leftU;

float saturate_cast(float v){
    if(v<0.0){
        v = 0.0;
    }else{
        if(v>255.0){
            v = 255.0;
        }
    }
    return v;
}

vec2 getMaxMin(float v1,float v2,float v3){
    vec2 maxMin;
    float min;
    float max;
    
    if(v1>v2)
	{
		max = v1;
		min = v2;
	}
	else
	{
		max = v2;
		min = v1;
	}
	if(v3>max)
	{
		max = v3;
	}
	if(v3<min)
	{
		min = v3;
	}
    
    maxMin.r = max;
    maxMin.g = min;
    
    return maxMin;
}

vec3 RGBtoHSL(float red,float green, float blue)
	{
            vec3 HSL;
            
            float var_R = red;                    
            float var_G = green;
            float var_B = blue;
            
            vec2 maxMin = getMaxMin(red,green,blue);
            
            float del_Max = maxMin.r - maxMin.g;
            HSL.b = ( maxMin.r + maxMin.g )/2.0;
            
            if ( del_Max == 0.0 )                    
            {
                HSL.r = 0.0;                               
                HSL.g = 0.0;
            }else{
                if ( HSL.b< 0.50 ) 
                    HSL.g = del_Max/( maxMin.r + maxMin.g );
                else   
                    HSL.g = del_Max / ( 2.0 - maxMin.r - maxMin.g );
                    
                float del_R = ( ( ( maxMin.r - var_R ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_G = ( ( ( maxMin.r - var_G ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_B = ( ( ( maxMin.r - var_B ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                
                if( var_R == maxMin.r ) 
                {
                    HSL.r = del_B - del_G;
                }else 
                {
                    if ( var_G == maxMin.r )
                    { 
                        HSL.r = ( 1.0 / 3.0 ) + del_R - del_B;
                    }
                    else 
                    {			
                        if ( var_B == maxMin.r ) 
                        {
                            HSL.r = ( 2.0 / 3.0 ) + del_G - del_R;
                        }
                    }
                }
                
                if ( HSL.r < 0.0 ) HSL.r += 1.0;
                if ( HSL.r > 1.0 ) HSL.r -= 1.0;
            }
            
            
            
            return HSL;
    }

float HSL_Hue_2_RGB(float v1,float v2,float vH )             //Function Hue_2_RGB
{
   if ( vH < 0.0 ) vH += 1.0;
   if ( vH > 1.0 ) vH -= 1.0;
   if ( ( 6.0 * vH ) < 1.0 ) return ( v1 + ( v2 - v1 ) * 6.0 * vH );
   if ( ( 2.0 * vH ) < 1.0 ) return ( v2 );
   if ( ( 3.0 * vH ) < 2.0 ) return ( v1 + ( v2 - v1 ) * ( ( 2.0 / 3.0 ) - vH ) * 6.0 );
   return v1;
}

vec3  HSLtoRGB(vec3 HSL)
{
    vec3 RGB;
	float H=HSL.r,S = HSL.g,L = HSL.b;
	if(HSL.g == 0.0)
	{
		RGB.r = L;
   		RGB.g = L;
   		RGB.b = L;
	}
	else
	{
        
		float var_1,var_2;
		if ( L < 0.5 ) 
			{
				var_2 = L * ( 1.0 + S );
			}
   		else           
			{
				var_2 = ( L + S ) - ( S * L );
			}
        
   		var_1 = 2.0 * L - var_2;
		RGB.r = HSL_Hue_2_RGB( var_1, var_2, H + ( 1.0 / 3.0 ) );
   		RGB.g = HSL_Hue_2_RGB( var_1, var_2, H );
   		RGB.b = HSL_Hue_2_RGB( var_1, var_2, H - ( 1.0 / 3.0 ) );
        
    }
    
    return RGB;
}

vec3 getRed(float x,float y){
	
	if(x<0.0){
		x = 0.0;
	}else if(x>=textureWidth){
		x = textureWidth-1.0;
	}
	
	if(y<0.0){
		y = 0.0;
	}else if(y>=textureHeight){
		y = textureHeight-1.0;
	}
	
	vec4 color = texture2D(sTexture, vec2(x/textureSize,y/textureSize));
	vec3 hsl = RGBtoHSL(color.r,color.g,color.b);
	hsl.x = 0.0;
	hsl.y = 100.0*hsl.y+5.0;
	
	if(hsl.y>100.0){
		hsl.y = 100.0;
	}
	
	hsl.y = hsl.y/100.0;
	
	hsl.z = 100.0*hsl.z-5.0;
	if(hsl.z<0.0){
		hsl.z = 0.0;
	}
	hsl.z = hsl.z/100.0;
	
	vec3 red = HSLtoRGB(hsl);
	return red;
}


vec3 getCyan(float x,float y){

	if(x<0.0){
		x = 0.0;
	}else if(x>=textureWidth){
		x = textureWidth-1.0;
	}
	
	if(y<0.0){
		y = 0.0;
	}else if(y>=textureHeight){
		y = textureHeight-1.0;
	}

	vec4 color = texture2D(sTexture, vec2(x/textureSize,y/textureSize));
	vec3 hsl = RGBtoHSL(color.r,color.g,color.b);
	hsl.x = 180.0/360.0;
	hsl.y = 100.0*hsl.y+5.0;
	
	if(hsl.y>100.0){
		hsl.y = 100.0;
	}
	
	hsl.y = hsl.y/100.0;
	
	vec3 cyan = HSLtoRGB(hsl);
	return cyan;
}

float hard(float m,float i){
	float result = 0.0;
	if(m < 128.0){
		result = 2.0 * m * i / 256.0;
	}else{
		result = 255.0 - (255.0 - 2.0 * (m - 128.0)) * (255.0 - i) / 256.0;
	}
	return result;
}

void main() {
	vec4 vColor = texture2D(sTexture, vTextureCoord);
	
	
	if(textureSize>0.0){
		
		float dl = textureWidth * leftU  / 200.0;
    	float dr = textureWidth * right / 200.0;
		
		float fXIndex = vTextureCoord.x*textureSize;
    	float fYIndex = vTextureCoord.y*textureSize;
    	
    	vec4 destColor = vColor;
    	
    	if(fXIndex<dr){
    		vec3 redColor = getRed((fXIndex+dl),fYIndex);
    		redColor.r = 255.0*redColor.r;
    		redColor.g = 255.0*redColor.g;
    		redColor.b = 255.0*redColor.b;
    	
    		vec3 cyan = getCyan((fXIndex+textureWidth-dr),fYIndex);
    		cyan.r = 255.0*cyan.r;
    		cyan.g = 255.0*cyan.g;
    		cyan.b = 255.0*cyan.b;
    	
    		float red = hard(redColor.r,cyan.r);
    		float green = hard(redColor.g,cyan.g);
    		float blue = hard(redColor.b,cyan.b);
    	
    		red = red/255.0;
    		green = green/255.0;
    		blue = blue/255.0;
    		destColor = vec4(red,green,blue,vColor.a);
    	}else if(fXIndex<textureWidth-dl){
    		vec3 redColor = getRed((fXIndex+dl),fYIndex);
    		redColor.r = 255.0*redColor.r;
    		redColor.g = 255.0*redColor.g;
    		redColor.b = 255.0*redColor.b;
    	
    		vec3 cyan = getCyan((fXIndex-dr),fYIndex);
    		cyan.r = 255.0*cyan.r;
    		cyan.g = 255.0*cyan.g;
    		cyan.b = 255.0*cyan.b;
    	
    		float red = hard(redColor.r,cyan.r);
    		float green = hard(redColor.g,cyan.g);
    		float blue = hard(redColor.b,cyan.b);
    	
    		red = red/255.0;
    		green = green/255.0;
    		blue = blue/255.0;
    		destColor = vec4(red,green,blue,vColor.a);
    		
    	}else{
    	
    		vec3 redColor = getRed((fXIndex-textureWidth+dl),fYIndex);
    		redColor.r = 255.0*redColor.r;
    		redColor.g = 255.0*redColor.g;
    		redColor.b = 255.0*redColor.b;
    	
    		vec3 cyan = getCyan((fXIndex-dr),fYIndex);
    		cyan.r = 255.0*cyan.r;
    		cyan.g = 255.0*cyan.g;
    		cyan.b = 255.0*cyan.b;
    	
    		float red = hard(redColor.r,cyan.r);
    		float green = hard(redColor.g,cyan.g);
    		float blue = hard(redColor.b,cyan.b);
    	
    		red = red/255.0;
    		green = green/255.0;
    		blue = blue/255.0;
    		destColor = vec4(red,green,blue,vColor.a);
    	}
    	
    	 vec4 dest = (fade*vColor+(100.0-fade)*destColor)/100.0;
    	 
    	 if(dest.r<0.0){
			dest.r = 0.0;
		 }else if(dest.r>1.0){
			dest.r = 1.0;
		 }
		
		 if(dest.g<0.0){
			dest.g = 0.0;
		 }else if(dest.g>1.0){
			dest.g = 1.0;
		 }
		
		 if(dest.b<0.0){
			dest.b = 0.0;
		 }else if(dest.b>1.0){
			dest.b = 1.0;
		 }
    	 
    	 gl_FragColor = dest;
    	
	}else{
		gl_FragColor = vColor;
	}
	
}