precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform float hue;
uniform float replaceDeltaSaturation;
uniform float fade;

vec2 getMaxMin(float v1,float v2,float v3){
    vec2 maxMin;
    float min;
    float max;
    
    if(v1>v2)
	{
		max = v1;
		min = v2;
	}
	else
	{
		max = v2;
		min = v1;
	}
	if(v3>max)
	{
		max = v3;
	}
	if(v3<min)
	{
		min = v3;
	}
    
    maxMin.r = max;
    maxMin.g = min;
    
    return maxMin;
}

vec3 RGBtoHSL(float red,float green, float blue)
	{
            vec3 HSL;
            
            float var_R = red;                    
            float var_G = green;
            float var_B = blue;
            
            vec2 maxMin = getMaxMin(red,green,blue);
            
            float del_Max = maxMin.r - maxMin.g;
            HSL.b = ( maxMin.r + maxMin.g )/2.0;
            
            if ( del_Max == 0.0 )                    
            {
                HSL.r = 0.0;                               
                HSL.g = 0.0;
            }else{
                if ( HSL.b< 0.50 ) 
                    HSL.g = del_Max/( maxMin.r + maxMin.g );
                else   
                    HSL.g = del_Max / ( 2.0 - maxMin.r - maxMin.g );
                    
                float del_R = ( ( ( maxMin.r - var_R ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_G = ( ( ( maxMin.r - var_G ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_B = ( ( ( maxMin.r - var_B ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                
                if( var_R == maxMin.r ) 
                {
                    HSL.r = del_B - del_G;
                }else 
                {
                    if ( var_G == maxMin.r )
                    { 
                        HSL.r = ( 1.0 / 3.0 ) + del_R - del_B;
                    }
                    else 
                    {			
                        if ( var_B == maxMin.r ) 
                        {
                            HSL.r = ( 2.0 / 3.0 ) + del_G - del_R;
                        }
                    }
                }
                
                if ( HSL.r < 0.0 ) HSL.r += 1.0;
                if ( HSL.r > 1.0 ) HSL.r -= 1.0;
            }
            
            
            
            return HSL;
    }

float HSL_Hue_2_RGB(float v1,float v2,float vH )             //Function Hue_2_RGB
{
   if ( vH < 0.0 ) vH += 1.0;
   if ( vH > 1.0 ) vH -= 1.0;
   if ( ( 6.0 * vH ) < 1.0 ) return ( v1 + ( v2 - v1 ) * 6.0 * vH );
   if ( ( 2.0 * vH ) < 1.0 ) return ( v2 );
   if ( ( 3.0 * vH ) < 2.0 ) return ( v1 + ( v2 - v1 ) * ( ( 2.0 / 3.0 ) - vH ) * 6.0 );
   return v1;
}

vec3  HSLtoRGB(vec3 HSL)
{
    vec3 RGB;
	float H=HSL.r,S = HSL.g,L = HSL.b;
	if(HSL.g == 0.0)
	{
		RGB.r = L;
   		RGB.g = L;
   		RGB.b = L;
	}
	else
	{
        
		float var_1,var_2;
		if ( L < 0.5 ) 
			{
				var_2 = L * ( 1.0 + S );
			}
   		else           
			{
				var_2 = ( L + S ) - ( S * L );
			}
        
   		var_1 = 2.0 * L - var_2;
		RGB.r = HSL_Hue_2_RGB( var_1, var_2, H + ( 1.0 / 3.0 ) );
   		RGB.g = HSL_Hue_2_RGB( var_1, var_2, H );
   		RGB.b = HSL_Hue_2_RGB( var_1, var_2, H - ( 1.0 / 3.0 ) );
        
    }
    
    return RGB;
}

void main() {
    vec4 color = texture2D(sTexture, vTextureCoord);
    
    vec3 HSL = RGBtoHSL(color.r,color.g,color.b);
    
    //float hue = 250.0;
    //float replaceDeltaSaturation = 20.0;
    
    float saturation = HSL.g;
    
    saturation += replaceDeltaSaturation/50.0;
    
    if(saturation>1.0){
        saturation = 1.0;
    }else{
        if(saturation<0.0){
            saturation = 0.0;
        }
    }
    
    HSL.r = hue/360.0;
    HSL.g = saturation;
    vec3 RGB = HSLtoRGB(HSL);
    vec4 destColor = vec4(RGB.r,RGB.g,RGB.b,1.0);
    
    gl_FragColor = (fade*color+(100.0-fade)*destColor)/100.0;
}