precision highp float;
varying vec2 vTextureCoord;
uniform sampler2D sTexture;

uniform float scaleFactor;
uniform float textureSize;
uniform float textureWidth;
uniform float textureHeight;

uniform float fade;
uniform float deltaHue;
uniform float deltaSaturation;
uniform float deltaLightness;

vec2 getMaxMin(float v1,float v2,float v3){
    vec2 maxMin;
    float min;
    float max;
    
    if(v1>v2)
	{
		max = v1;
		min = v2;
	}
	else
	{
		max = v2;
		min = v1;
	}
	if(v3>max)
	{
		max = v3;
	}
	if(v3<min)
	{
		min = v3;
	}
    
    maxMin.r = max;
    maxMin.g = min;
    
    return maxMin;
}

vec3 RGBtoHSL(float red,float green, float blue)
	{
            vec3 HSL;
            
            float var_R = red;                    
            float var_G = green;
            float var_B = blue;
            
            vec2 maxMin = getMaxMin(red,green,blue);
            
            float del_Max = maxMin.r - maxMin.g;
            HSL.b = ( maxMin.r + maxMin.g )/2.0;
            
            if ( del_Max == 0.0 )                    
            {
                HSL.r = 0.0;                               
                HSL.g = 0.0;
            }else{
                if ( HSL.b< 0.50 ) 
                    HSL.g = del_Max/( maxMin.r + maxMin.g );
                else   
                    HSL.g = del_Max / ( 2.0 - maxMin.r - maxMin.g );
                    
                float del_R = ( ( ( maxMin.r - var_R ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_G = ( ( ( maxMin.r - var_G ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                float del_B = ( ( ( maxMin.r - var_B ) / 6.0 ) + ( del_Max / 2.0 ) ) / del_Max;
                
                if( var_R == maxMin.r ) 
                {
                    HSL.r = del_B - del_G;
                }else 
                {
                    if ( var_G == maxMin.r )
                    { 
                        HSL.r = ( 1.0 / 3.0 ) + del_R - del_B;
                    }
                    else 
                    {			
                        if ( var_B == maxMin.r ) 
                        {
                            HSL.r = ( 2.0 / 3.0 ) + del_G - del_R;
                        }
                    }
                }
                
                if ( HSL.r < 0.0 ) HSL.r += 1.0;
                if ( HSL.r > 1.0 ) HSL.r -= 1.0;
            }
            
            
            
            return HSL;
    }

float HSL_Hue_2_RGB(float v1,float v2,float vH )             //Function Hue_2_RGB
{
   if ( vH < 0.0 ) vH += 1.0;
   if ( vH > 1.0 ) vH -= 1.0;
   if ( ( 6.0 * vH ) < 1.0 ) return ( v1 + ( v2 - v1 ) * 6.0 * vH );
   if ( ( 2.0 * vH ) < 1.0 ) return ( v2 );
   if ( ( 3.0 * vH ) < 2.0 ) return ( v1 + ( v2 - v1 ) * ( ( 2.0 / 3.0 ) - vH ) * 6.0 );
   return v1;
}

vec3  HSLtoRGB(vec3 HSL)
{
    vec3 RGB;
	float H=HSL.r,S = HSL.g,L = HSL.b;
	if(HSL.g == 0.0)
	{
		RGB.r = L;
   		RGB.g = L;
   		RGB.b = L;
	}
	else
	{
        
		float var_1,var_2;
		if ( L < 0.5 ) 
			{
				var_2 = L * ( 1.0 + S );
			}
   		else           
			{
				var_2 = ( L + S ) - ( S * L );
			}
        
   		var_1 = 2.0 * L - var_2;
		RGB.r = HSL_Hue_2_RGB( var_1, var_2, H + ( 1.0 / 3.0 ) );
   		RGB.g = HSL_Hue_2_RGB( var_1, var_2, H );
   		RGB.b = HSL_Hue_2_RGB( var_1, var_2, H - ( 1.0 / 3.0 ) );
        
    }
    
    return RGB;
}

void main() {
	vec4 vColor = texture2D(sTexture, vTextureCoord);
	
	float redSum =  vColor.r;
 	float greenSum =  vColor.g;
 	float blueSum =  vColor.b;
   
    float fXIndex = vTextureCoord.x*textureSize;
    float fYIndex = vTextureCoord.y*textureSize;
   
    if( ! ( fYIndex < scaleFactor || fYIndex > textureHeight - scaleFactor || 
            fXIndex < scaleFactor || fXIndex > textureWidth - scaleFactor ))
    {
    	float redXSum = 0.0;
        float redYSum = 0.0;
    
        float greenXSum = 0.0;
        float greenYSum = 0.0;
    
        float blueXSum = 0.0;
        float blueYSum = 0.0;
        
        float fTempX = ( fXIndex  -scaleFactor + 0.5 ) / textureSize ;
        float fTempY = ( fYIndex  -scaleFactor + 0.5 ) / textureSize ;
        vec4 vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // X Direction
        redXSum = redXSum -  vTempColor.r;
        greenXSum = greenXSum -  vTempColor.g;
        blueXSum = blueXSum -  vTempColor.b;
        
        // Y Direction
        redYSum = redYSum + vTempColor.r;
        greenYSum = greenYSum + vTempColor.g;
        blueYSum = blueYSum + vTempColor.b;
        
        fTempX = ( fXIndex +scaleFactor + 0.5 ) / textureSize ;
        fTempY = ( fYIndex  -scaleFactor + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // X Direction
        redXSum = redXSum + vTempColor.r;
        greenXSum = greenXSum + vTempColor.g;
        blueXSum = blueXSum + vTempColor.b;
        
        // Y Direction
        redYSum = redYSum + vTempColor.r;
        greenYSum = greenYSum + vTempColor.g;
        blueYSum = blueYSum + vTempColor.b;
        
        fTempX = ( fXIndex -scaleFactor + 0.5 ) / textureSize ;
        fTempY = ( fYIndex  + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // X Direction
        redXSum = redXSum-2.0*vTempColor.r;
        greenXSum = greenXSum-2.0*vTempColor.g;
        blueXSum = blueXSum-2.0*vTempColor.b;
        
        fTempX = ( fXIndex +scaleFactor + 0.5 ) / textureSize ;
        fTempY = ( fYIndex  + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
   	 	blueSum += vTempColor.b;
        
        // X Direction
        redXSum = redXSum+2.0*vTempColor.r;
        greenXSum = greenXSum+2.0*vTempColor.g;
        blueXSum = blueXSum+2.0*vTempColor.b;
        
        
        fTempX = ( fXIndex -scaleFactor + 0.5 ) / textureSize ;
        fTempY = ( fYIndex +scaleFactor + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // X Direction
        redXSum = redXSum - vTempColor.r;
        greenXSum = greenXSum - vTempColor.g;
        blueXSum = blueXSum - vTempColor.b;
        
        // Y Direction
        redYSum = redYSum - vTempColor.r;
        greenYSum = greenYSum - vTempColor.g;
        blueYSum = blueYSum - vTempColor.b;
        
        
        fTempX = ( fXIndex +scaleFactor + 0.5 ) / textureSize ;
        fTempY = ( fYIndex +scaleFactor + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
         // X Direction
        redXSum = redXSum + vTempColor.r;
        greenXSum = greenXSum + vTempColor.g;
        blueXSum = blueXSum + vTempColor.b;
        
        // Y Direction
        redYSum = redYSum - vTempColor.r;
        greenYSum = greenYSum - vTempColor.g;
        blueYSum = blueYSum - vTempColor.b;
        
        
        fTempX = ( fXIndex  + 0.5 ) / textureSize ;
        fTempY = ( fYIndex -scaleFactor + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // Y Direction
        redYSum = redYSum + 2.0*vTempColor.r;
        greenYSum = greenYSum + 2.0*vTempColor.g;
        blueYSum = blueYSum + 2.0*vTempColor.b;
        
        fTempX = ( fXIndex  + 0.5 ) / textureSize ;
        fTempY = ( fYIndex +scaleFactor + 0.5 ) / textureSize ;
        vTempColor = texture2D(sTexture, vec2(fTempX,fTempY));
    	redSum += vTempColor.r;
    	greenSum += vTempColor.g;
    	blueSum += vTempColor.b;
        
        // Y Direction
        redYSum = redYSum - 2.0*vTempColor.r;
        greenYSum = greenYSum - 2.0*vTempColor.g;
        blueYSum = blueYSum - 2.0*vTempColor.b;
        
        /*float fRedTotalSum = sqrt((redXSum*redXSum+redYSum*redYSum));
        
        float fGreenTotalSum = sqrt((greenXSum*greenXSum+greenYSum*greenYSum));
        
        float fBlueTotalSum = sqrt((blueXSum*blueXSum+blueYSum*blueYSum));
        
        
        float oneFloat = float(1.0);
        float zeroFloat = float(0.0);
        
        if(fRedTotalSum<zeroFloat){
        	fRedTotalSum = zeroFloat;
        }
        
        if(fRedTotalSum>oneFloat){
        	fRedTotalSum = oneFloat;
        }
        
        if(fGreenTotalSum<zeroFloat){
        	fGreenTotalSum = zeroFloat;
        }
        
        if(fGreenTotalSum>oneFloat){
        	fGreenTotalSum = oneFloat;
        }
        
        
        if(fBlueTotalSum<zeroFloat){
        	fBlueTotalSum = zeroFloat;
        }
        
        if(fBlueTotalSum>oneFloat){
        	fBlueTotalSum = oneFloat;
        }
        
        
        fRedTotalSum = 1.0 - fRedTotalSum;
        fGreenTotalSum = 1.0 - fGreenTotalSum;
        fBlueTotalSum = 1.0 - fBlueTotalSum;*/
        
        //**************
        
        float red = redSum/9.0;
        float green = greenSum/9.0;
        float blue = blueSum/9.0;
        
        
        vec3 HSL = RGBtoHSL(red,green,blue);
        
        float hue = HSL.r*360.0;
 
 
 		if(hue>=(360.0-deltaHue) || hue<=deltaHue){
    		hue = 0.0;
 		}else{
 			hue = hue/deltaHue;
    		float index = floor(hue);
    		float minHue = index*deltaHue;
    		float maxHue = minHue+deltaHue;
    		hue = (minHue+maxHue)/2.0;
 		}
 		
 		hue /= 360.0;
 
 		HSL.r = hue;
 
 		
 		float saturation = HSL.g*100.0;
		saturation = saturation/deltaSaturation;

 		float saturationIndex = floor(saturation);
 		float minSaturation = saturationIndex*deltaSaturation;
 		float maxSaturation = minSaturation+deltaSaturation;
 
 		saturation = (minSaturation+maxSaturation)/2.0;

 
 		if(saturation>100.0){
    		saturation = 100.0;
 		}
 
 		HSL.g = saturation/100.0;
 
 		float lightness = HSL.b*100.0;
 		lightness = lightness/deltaLightness;
 
 		float lightnessIndex = floor(lightness);
 		float minLightness = lightnessIndex*deltaLightness;
 		float maxLightness = minLightness+deltaLightness;
 
 		lightness = (minLightness+maxLightness)/2.0;
 
 		if(lightness>100.0){
    		lightness = 100.0;
 		}
 
 		HSL.b = lightness/100.0;
        
        vec3 RGB = HSLtoRGB(HSL);
        
        vColor.r = RGB.r;
        vColor.g = RGB.g;
        vColor.b = RGB.b;
    
        
        
        
    }
	
    gl_FragColor = vColor;
}